library(tuneR)
library(seewave)
library(zoo)
library(dtw)

orca <- readWave("orca.wav")
# wav a 44100 Hz
orca

wl <- 1024
ovlp <- 87.5
# graficar espectrograma
fspec <- spectro(orca,
                 osc=TRUE,
                 wl=wl, 
                 wn="hanning",
                 ovlp=ovlp,
                 zp=wl,
                 collevels=seq(-100, 0, 1),
                 palette=temp.colors)


# recortamos las cuatro notas
note1 <- cutw(orca, from=0.10, to=1.25, output="Wave")
note2 <- cutw(orca, from=1.55, to=2.29, output="Wave")
note3 <- cutw(orca, from=4.75, to=6.00, output="Wave")
note4 <- cutw(orca, from=6.85, to=7.60, output="Wave")

###### DOMINANT FREQUENCY NOTE 1 ######
# espectrograma de la nota
spectro(note1, wl=wl, ovlp=ovlp, 
        palette=reverse.gray.colors.2, 
        norm=TRUE, flim=c(0,5))

# frecuencia dominante
df1 <- dfreq(note1, 
             wl=wl, ovlp=ovlp,
             clip=0.4,
             bandpass=c(1500, 3200),
             plot=FALSE)

# gráfico de frequencia dominante 
points(df1, col='blue', pch=19)
# interpolacion de valores NA
df1_na <- na.approx(df1)
points(df1_na, col='red', pch=19)

###### DOMINANT FREQUENCY NOTE 2 ######
# espectrograma de la nota
spectro(note2, wl=wl, ovlp=ovlp, 
        palette=reverse.gray.colors.2, 
        norm=TRUE, flim=c(0,5))

# frecuencia dominante
df2 <- dfreq(note2, 
             wl=wl, ovlp=ovlp,
             clip=0.3,
             bandpass=c(1900, 3600),
             plot=FALSE)

# gráfico de frequencia dominante 
points(df2, col='blue', pch=19)
# puntos mal estimados se definen NA
df2[31,2] <- NA
df2[33,2] <- NA
# interpolacion de valores NA
df2_na <- na.approx(df2)
points(df2_na, col='red', pch=19)

###### DOMINANT FREQUENCY NOTE 3 ######
# espectrograma de la nota
spectro(note3, wl=wl, ovlp=ovlp, 
        palette=reverse.gray.colors.2, 
        norm=TRUE, flim=c(0,5))

# frecuencia dominante
df3 <- dfreq(note3, 
             wl=wl, ovlp=ovlp,
             clip=0.2,
             bandpass=c(1800, 3400),
             plot=FALSE)

# gráfico de frequencia dominante 
points(df3, col='blue', pch=19)
# interpolacion de valores NA
df3_na <- na.approx(df3)
points(df3_na, col='red', pch=19)

###### DOMINANT FREQUENCY NOTE 4 ######
# espectrograma de la nota
spectro(note4, wl=wl, ovlp=ovlp, 
        palette=reverse.gray.colors.2, 
        norm=TRUE, flim=c(0,5))

# frecuencia dominante
df4 <- dfreq(note4, 
             wl=wl, ovlp=ovlp,
             clip=0.3,
             bandpass=c(1900, 3800),
             plot=FALSE)

# gráfico de frequencia dominante 
points(df4, col='blue', pch=19)
# puntos mal estimados se definen NA
df4[23,2] <- NA
df4[32,2] <- NA
df4[35,2] <- NA
df4[36,2] <- NA
df4[37,2] <- NA
df4[43,2] <- NA
# interpolacion de valores NA
df4_na <- na.approx(df4)
points(df4_na, col='red', pch=19)

# ============ DTW =============

# omitir valores NA
df1_ona = na.omit(df1_na)
df2_ona = na.omit(df2_na)
df3_ona = na.omit(df3_na)
df4_ona = na.omit(df4_na)

# solo valores de frecuencia
df1_flt <- df1_ona[,2]
df2_flt <- df2_ona[,2]
df3_flt <- df3_ona[,2]
df4_flt <- df4_ona[,2]

# alineamiento notas 1 y 2
dtw12 <- dtw(df1_flt, df2_flt, keep=TRUE)
plot(dtw12, type="twoway", offset=1)
# distancia DTW de notas 1 y 2
dtw12$distance

# alineamiento notas 1 y 3
dtw13 <- dtw(df1_flt, df3_flt, keep=TRUE)
plot(dtw13, type="twoway", offset=1)
# distancia DTW de notas 1 y 3
dtw13$distance

# alineamiento notas 2 y 3
dtw23 <- dtw(df2_flt, df3_flt, keep=TRUE)
plot(dtw23, type="twoway", offset=1)
# distancia DTW de notas 2 y 3
dtw23$distance

# alineamiento notas 2 y 4
dtw24 <- dtw(df2_flt, df4_flt, keep=TRUE)
plot(dtw24, type="twoway", offset=1)
# distancia DTW de notas 2 y 4
dtw24$distance