library(monitoR)
library(seewave)

pipi <- readWave('./pipistrellus.wav')
temp <- readWave('./templates.wav', 
                 from=6.5,to=9,units="seconds")

# espectrogramas
layout(matrix(1:2, nc=1))
viewSpec(pipi)
viewSpec(temp)

# espectrograma de templates
spectro(temp, wl=1024, ovlp=50, 
        palette=reverse.gray.colors.2, 
        norm=TRUE)

# se crean los templates
temp1 <- makeCorTemplate('./templates.wav', 
                         t.lim = c(6.5, 7), 
                         frq.lim = c(2, 8), 
                         name = "temp1")
temp2 <- makeCorTemplate('./templates.wav', 
                         t.lim = c(7.5, 8), 
                         frq.lim = c(2, 8), 
                         name = "temp2")
temp3 <- makeCorTemplate('./templates.wav',  
                         t.lim = c(8.5, 9), 
                         frq.lim = c(2, 8), 
                         name = "temp3")

# se combinan los templates 
ctemps <- combineCorTemplates(temp1,temp2,temp3)
ctemps
plot(ctemps)

# correlacion entre el espectro de la señal original y cada template
cscores <- corMatch('./pipistrellus.wav', ctemps)

# se buscan picos en la correlacion de los espectros
peaks <- findPeaks(cscores)
peaks
# se muestran las detecciones
getDetections(peaks)

# grafico de las detecciones con un solo template
plot(peaks, 
     which.one="temp3", 
     legend=FALSE, 
     hit.marker="points",
     color='red')
